//--------------------------------------------------------------
// File     : stm32_ub_bma180.c
// Datum    : 28.09.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : STM32_UB_SPI4
// Funktion : 3Achs Beschleunigungs-Sensor (BMA180)
//            Auflsung = 14bit
//
// Hinweis  : SPI-Max = 10MHz (Mode-3 MSB)
//            Interrupt-Leitung wird nicht benutzt
//
// PB2 = ChipSelect
// PE2 = CLK
// PE5 = MISO (SDO)
// PE6 = MOSI (SDI)
//
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_bma180.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_BMA180_initIO(void);
void P_BMA180_CS(BitAction wert);
uint8_t P_BMA180_readAdr(uint8_t adr);
void P_BMA180_writeAdr(uint8_t adr, uint8_t wert);
void P_BMA180_readRAW(void);
void P_BMA180_calcData(void);
void P_BMA180_setScale(BMA180_SCALE_t scale);
void P_BMA180_setFilter(BMA180_FILTER_t filter);


//--------------------------------------------------------------
// Init vom BMA180
// scale : [SCALE_1G,SCALE_1G5,SCALE_2G,...,SCALE_16G]
// Return_wert :
//  -> ERROR   , BMA180 nicht gefunden
//  -> SUCCESS , init vom BMA180 ok
//--------------------------------------------------------------
ErrorStatus UB_BMA180_Init(BMA180_SCALE_t scale)
{
  ErrorStatus ret_wert=ERROR;
  uint8_t wert;

  // init aller Variabeln
  BMA180.x_achse=0;
  BMA180.y_achse=0;
  BMA180.z_achse=0;
  BMA180.temperatur=0;

  BMA180_RAW.x_achse=0;
  BMA180_RAW.y_achse=0;
  BMA180_RAW.z_achse=0;
  BMA180_RAW.temperatur=0;
  BMA180_RAW.faktor=BMA180_1G_FAKTOR;

  // init aller IO-Pins
  P_BMA180_initIO();

  // init der SPI-Schnittstelle (MODE-3)
  UB_SPI4_Init(SPI4_MODE_3_MSB);

  // check ob ID vom BMA180 ok ist
  wert=P_BMA180_readAdr(BMA180_REG_ID);
  if(wert==BMA180_ID_VALUE) {
    P_BMA180_setScale(scale);
    UB_BMA180_SetFilter(FILTER_1200Hz);
    ret_wert=SUCCESS;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Auslesen vom BMA180 und fuellen der Struktur "BMA180"
// (die Funktion muss zyklisch aufgerufen werden)
//   BMA180.x_achse => Beschleunigung der X-Achse (in +/- mg)
//   BMA180.y_achse => Beschleunigung der Y-Achse (in +/- mg)
//   BMA180.z_achse => Beschleunigung der Z-Achse (in +/- mg)
//--------------------------------------------------------------
void UB_BMA180_Read(void)
{
  // auslesen der Rohdaten
  P_BMA180_readRAW();

  // umrechnen der Werte
  P_BMA180_calcData();
}


//--------------------------------------------------------------
// einstellen vom Filter
// filter : [FILTER_10Hz, FILTER_20Hz, ... , FILTER_BAND_PASS]
//--------------------------------------------------------------
void UB_BMA180_SetFilter(BMA180_FILTER_t filter)
{
  P_BMA180_setFilter(filter);
}


//--------------------------------------------------------------
// interne Funktion
// init aller IO-Pins
//--------------------------------------------------------------
void P_BMA180_initIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  // Init der Chip-Select-Leitung
  RCC_AHB1PeriphClockCmd(BMA180_CS_GPIO_CLK,ENABLE);

  GPIO_InitStructure.GPIO_Pin = BMA180_CS_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(BMA180_CS_GPIO_PORT, &GPIO_InitStructure);

  // ChipSelect auf Hi
  P_BMA180_CS(Bit_SET);
}


//--------------------------------------------------------------
// interne Funktion
// Pegel von ChipSelect einstellen
//--------------------------------------------------------------
void P_BMA180_CS(BitAction wert) {
  if(wert==Bit_RESET) {
    BMA180_CS_GPIO_PORT->BSRRH = BMA180_CS_PIN;
  }
  else {
    BMA180_CS_GPIO_PORT->BSRRL = BMA180_CS_PIN;
  }
}


//--------------------------------------------------------------
// interne Funktion
// auslesen von einem Byte aus einer Adresse
//--------------------------------------------------------------
uint8_t P_BMA180_readAdr(uint8_t adr)
{
  uint8_t ret_wert=0;

  // Mode=read
  adr|=0x80;

  // CS auf Lo
  P_BMA180_CS(Bit_RESET);

  // adresse senden
  UB_SPI4_SendByte(adr);

  // daten auslesen (0x55 = Dummy-Byte)
  ret_wert=UB_SPI4_SendByte(0x55);

  // CS auf Hi
  P_BMA180_CS(Bit_SET);

  return(ret_wert);
}


//--------------------------------------------------------------
// interne Funktion
// schreiben von einem Byte in eine Adresse
//--------------------------------------------------------------
void P_BMA180_writeAdr(uint8_t adr, uint8_t wert)
{
  uint32_t n;

  // Mode=write
  adr&=~0x80;

  // CS auf Lo
  P_BMA180_CS(Bit_RESET);

  // adresse senden
  UB_SPI4_SendByte(adr);

  // kleine Pause
  for(n=0;n<BMA180_WRITE_DELAY;n++);

  // wert senden
  UB_SPI4_SendByte(wert);

  // CS auf Hi
  P_BMA180_CS(Bit_SET);

  // kleine Pause
  for(n=0;n<BMA180_WRITE_DELAY;n++);
}


//--------------------------------------------------------------
// interne Funktion
// auslesen der Roh-Daten vom BMA180
//--------------------------------------------------------------
void P_BMA180_readRAW(void)
{
  uint8_t wert_lo, wert_hi;
  uint16_t wert;

  // X-Achse
  wert_lo=P_BMA180_readAdr(BMA180_REG_XLSB);  // 6bit
  wert_hi=P_BMA180_readAdr(BMA180_REG_XMSB);  // 8bit
  wert=(wert_hi<<8);
  wert|=wert_lo;
  wert=(wert>>2); // 14bit
  BMA180_RAW.x_achse=wert;

  // Y-Achse
  wert_lo=P_BMA180_readAdr(BMA180_REG_YLSB);  // 6bit
  wert_hi=P_BMA180_readAdr(BMA180_REG_YMSB);  // 8bit
  wert=(wert_hi<<8);
  wert|=wert_lo;
  wert=(wert>>2); // 14bit
  BMA180_RAW.y_achse=wert;

  // Z-Achse
  wert_lo=P_BMA180_readAdr(BMA180_REG_ZLSB);  // 6bit
  wert_hi=P_BMA180_readAdr(BMA180_REG_ZMSB);  // 8bit
  wert=(wert_hi<<8);
  wert|=wert_lo;
  wert=(wert>>2); // 14bit
  BMA180_RAW.z_achse=wert;

  // Temperatur
  wert_lo=P_BMA180_readAdr(BMA180_REG_TEMP);  // 8bit
  BMA180_RAW.temperatur=wert_lo;
}


//--------------------------------------------------------------
// interne Funktion
// umrechnen der Roh-Daten in G-Werte und Temperatur
//--------------------------------------------------------------
void P_BMA180_calcData(void)
{
  uint16_t wert;
  int16_t beschleunigung,temp;

  // X-Achse
  wert=BMA180_RAW.x_achse;
  if((wert&0x2000)!=0) {
    // negativ
    wert=((0xFFFF^wert)&0x1FFF);
    wert++;
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.x_achse=0-beschleunigung;
  }
  else {
    // positiv
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.x_achse=beschleunigung;
  }

  // Y-Achse
  wert=BMA180_RAW.y_achse;
  if((wert&0x2000)!=0) {
    // negativ
    wert=((0xFFFF^wert)&0x1FFF);
    wert++;
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.y_achse=0-beschleunigung;
  }
  else {
    // positiv
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.y_achse=beschleunigung;
  }

  // Z-Achse
  wert=BMA180_RAW.z_achse;
  if((wert&0x2000)!=0) {
    // negativ
    wert=((0xFFFF^wert)&0x1FFF);
    wert++;
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.z_achse=0-beschleunigung;
  }
  else {
    // positiv
    beschleunigung=(int16_t)(BMA180_RAW.faktor*wert);
    BMA180.z_achse=beschleunigung;
  }

  // Temperatur
  wert=BMA180_RAW.temperatur;
  if((wert&0x80)!=0) {
    // negativ
    wert=((0xFFFF^wert)&0x007F);
    wert++;
    temp=(int16_t)(wert*0.5);
    BMA180.temperatur=0-temp+24; // 0=24C
  }
  else {
	// positiv
    temp=(int16_t)(BMA180_RAW.temperatur*0.5);
    BMA180.temperatur=temp+24; // 0=24C
  }
}


//--------------------------------------------------------------
// interne Funktion
// einstellung der Auflsung
//--------------------------------------------------------------
void P_BMA180_setScale(BMA180_SCALE_t scale)
{
  uint8_t ctrl_wert;
  uint8_t scale_wert, wert;

  // write enable setzen
  ctrl_wert=P_BMA180_readAdr(BMA180_REG_CTRL0);
  ctrl_wert|=0x10;
  P_BMA180_writeAdr(BMA180_REG_CTRL0, ctrl_wert);

  // default
  BMA180_RAW.faktor=BMA180_1G_FAKTOR;
  wert=0x00;

  if(scale==SCALE_1G) {
    BMA180_RAW.faktor=BMA180_1G_FAKTOR;
    wert=0x00;
  }
  else if(scale==SCALE_1G5) {
    BMA180_RAW.faktor=BMA180_1G5_FAKTOR;
    wert=0x02;
  }
  else if(scale==SCALE_2G) {
    BMA180_RAW.faktor=BMA180_2G_FAKTOR;
    wert=0x04;
  }
  else if(scale==SCALE_3G) {
    BMA180_RAW.faktor=BMA180_3G_FAKTOR;
    wert=0x06;
  }
  else if(scale==SCALE_4G) {
    BMA180_RAW.faktor=BMA180_4G_FAKTOR;
    wert=0x08;
  }
  else if(scale==SCALE_8G) {
    BMA180_RAW.faktor=BMA180_8G_FAKTOR;
    wert=0x0A;
  }
  else if(scale==SCALE_16G) {
    BMA180_RAW.faktor=BMA180_16G_FAKTOR;
    wert=0x0C;
  }

  // neue Range setzen
  scale_wert=P_BMA180_readAdr(BMA180_REG_RANGE);
  scale_wert&=0xF1;
  scale_wert|=wert;
  P_BMA180_writeAdr(BMA180_REG_RANGE, scale_wert);
}


//--------------------------------------------------------------
// interne Funktion
// einstellung vom Filter
//--------------------------------------------------------------
void P_BMA180_setFilter(BMA180_FILTER_t filter)
{
  uint8_t ctrl_wert;
  uint8_t filter_wert, wert;

  // write enable setzen
  ctrl_wert=P_BMA180_readAdr(BMA180_REG_CTRL0);
  ctrl_wert|=0x10;
  P_BMA180_writeAdr(BMA180_REG_CTRL0, ctrl_wert);

  // default
  wert=0x70;

  if(filter==FILTER_10Hz) wert=0x00;
  if(filter==FILTER_20Hz) wert=0x10;
  if(filter==FILTER_40Hz) wert=0x20;
  if(filter==FILTER_75Hz) wert=0x30;
  if(filter==FILTER_150Hz) wert=0x40;
  if(filter==FILTER_300Hz) wert=0x50;
  if(filter==FILTER_600Hz) wert=0x60;
  if(filter==FILTER_1200Hz) wert=0x70;
  if(filter==FILTER_HIGH_1Hz) wert=0x80;
  if(filter==FILTER_BAND_PASS) wert=0x90;

  // neuen Filter setzen
  filter_wert=P_BMA180_readAdr(BMA180_REG_FILTER);
  filter_wert&=0x0F;
  filter_wert|=wert;
  P_BMA180_writeAdr(BMA180_REG_FILTER, filter_wert);
}


